@@ -1,16 +1,25 @@
-# -*- coding: utf-8 -*-
+import os
 import string
-from random import choice
+from random import choice, randrange, shuffle
+
+from django.conf import global_settings
 from django.contrib import admin
 from django.contrib.admin.sites import AlreadyRegistered
-from django.contrib.auth.models import Group, Permission
+from django.contrib.auth import authenticate
+from django.contrib.auth.models import Group, Permission, User
+from django.contrib.contenttypes.models import ContentType
 from django.forms import BaseForm
+from django.test.testcases import TestCase
 from django_dynamic_fixture import G
+from django_dynamic_fixture.fixture_algorithms.random_fixture import \
+    RandomDataFixture
+
 from adminactions.exceptions import ActionInterrupted
-from adminactions.signals import adminaction_requested, adminaction_start, adminaction_end
+from adminactions.signals import (adminaction_end, adminaction_requested,
+                                  adminaction_start,)
 
 
-class admin_register(object):
+class admin_register:
     def __init__(self, model, model_admin=None, unregister=False):
         self.model = model
         self.model_admin = model_admin
@@ -58,16 +67,16 @@ def get_group(name=None, permissions=None):
             app_label, codename = permission_name.split('.')
         except ValueError:
             raise ValueError("Invalid permission name `{0}`".format(permission_name))
-        try:
-            permission = Permission.objects.get(content_type__app_label=app_label, codename=codename)
-        except Permission.DoesNotExist:
-            raise Permission.DoesNotExist('Permission `{0}` does not exists', permission_name)
-
+        __, model_name = codename.rsplit('_', 1)
+        ct = ContentType.objects.get(app_label__iexact=app_label,
+                                     model__iexact=model_name)
+        permission = Permission.objects.get(content_type=ct,
+                                            codename=codename)
         group.permissions.add(permission)
     return group
 
 
-class user_grant_permission(object):
+class user_grant_permission:
     def __init__(self, user, permissions=None):
         self.user = user
         self.permissions = permissions
@@ -96,9 +105,10 @@ class user_grant_permission(object):
         return self.__exit__()
 
 
-class SelectRowsMixin(object):
+class SelectRowsMixin:
     _selected_rows = []
     _selected_values = []
+    csrf_checks = False
 
     def _select_rows(self, form, selected_rows=None):
         if selected_rows is None:
@@ -112,7 +122,7 @@ class SelectRowsMixin(object):
                 self._selected_values.append(int(chk.value))
 
 
-class CheckSignalsMixin(object):
+class CheckSignalsMixin:
     MESSAGE = 'Action Interrupted Test'
 
     def test_signal_sent(self):
@@ -199,3 +209,90 @@ class CheckSignalsMixin(object):
             self.assertTrue(myhandler.invoked)
         finally:
             adminaction_end.disconnect(myhandler, sender=self.sender_model)
+
+
+def ipaddress(not_valid=None):
+    """
+        returns a string representing a random ip address
+
+    :param not_valid: if passed must be a list of integers representing valid class A netoworks that must be ignored
+    """
+    not_valid_class_A = not_valid or []
+
+    class_a = [r for r in range(1, 256) if r not in not_valid_class_A]
+    shuffle(class_a)
+    first = class_a.pop()
+
+    return ".".join([str(first), str(randrange(1, 256)),
+                     str(randrange(1, 256)), str(randrange(1, 256))])
+
+
+class DataFixtureClass(RandomDataFixture):  # it can inherit of SequentialDataFixture, RandomDataFixture etc.
+    def genericipaddressfield_config(self, field, key):  # method name must have the format: FIELDNAME_config
+        return ipaddress()
+
+
+TEST_TEMPLATES_DIR = os.path.join(os.path.dirname(__file__),
+                                  os.pardir, 'tests', 'templates')
+
+SETTINGS = {'TEMPLATE_DIRS': [TEST_TEMPLATES_DIR],
+            'AUTHENTICATION_BACKENDS': ('django.contrib.auth.backends.ModelBackend',),
+            'TEMPLATE_LOADERS': ('django.template.loaders.filesystem.Loader',
+                                 'django.template.loaders.app_directories.Loader'),
+            # 'AUTH_PROFILE_MODULE': None,
+            'TEMPLATE_CONTEXT_PROCESSORS': ("django.contrib.auth.context_processors.auth",
+                                            "django.core.context_processors.debug",
+                                            "django.core.context_processors.i18n",
+                                            "django.core.context_processors.media",
+                                            "django.core.context_processors.static",
+                                            "django.core.context_processors.request",
+                                            "django.core.context_processors.tz",
+                                            "django.contrib.messages.context_processors.messages")}
+
+SETTINGS['MIDDLEWARE'] = global_settings.MIDDLEWARE
+
+ADMIN = 'sax'
+USER = 'user'
+PWD = '123'
+USER_EMAIL = 'user@moreply.org'
+
+
+class BaseTestCaseMixin:
+    fixtures = ['adminactions.json', ]
+
+    def setUp(self):
+        super().setUp()
+        self.sett = self.settings(**SETTINGS)
+        self.sett.enable()
+        self.login()
+
+    def tearDown(self):
+        self.sett.disable()
+
+    def login(self, username='user_00', password='123'):
+        user = User.objects.get(username=username)
+        try:
+            self.client.force_login(user)
+        except AttributeError:  # for Django<1.8
+            logged = self.client.login(username=username, password=password)
+            assert logged, 'Unable login with credentials'
+        self._user = authenticate(username=username, password=password)
+
+    def add_permission(self, *perms, **kwargs):
+        """ add the right permission to the user """
+        target = kwargs.pop('user', self._user)
+        if hasattr(target, '_perm_cache'):
+            del target._perm_cache
+        for perm_name in perms:
+            app_label, code = perm_name.split('.')
+            if code == '*':
+                perms = Permission.objects.filter(content_type__app_label=app_label)
+            else:
+                perms = Permission.objects.filter(codename=code, content_type__app_label=app_label)
+            target.user_permissions.add(*perms)
+
+        target.save()
+
+
+class BaseTestCase(BaseTestCaseMixin, TestCase):
+    pass
